@@ -111,7 +111,7 @@ GdkPixbufFormat *_gdk_pixbuf_get_format (GdkPixbufModule *image_module);
 
 #endif /* GDK_PIXBUF_ENABLE_BACKEND */
 
-GdkPixbuf * _gdk_pixbuf_new_from_resource_try_mmap (const char *resource_path);
+GdkPixbuf * _gdk_pixbuf_new_from_resource_try_pixdata (const char *resource_path);
 GdkPixbufLoader *_gdk_pixbuf_loader_new_with_filename (const char *filename);
 
 void _gdk_pixbuf_init_gettext (void);
